/**
 * cli_formatters.js
 *
 * Dedicated formatters for CLI output of various data types
 * Converts raw data into formatted CLI-friendly displays
 */
const chalk = require('chalk');

/**
 * Format Sigma statistics for CLI display
 * 
 * @param {Object} stats - The statistics object
 * @returns {Object} Formatted stats ready for CLI display
 */
function formatSigmaStats(stats) {
  if (!stats) {
    return { error: 'No statistics data available' };
  }
  
  // Format date
  const formatDate = (dateString) => {
    if (!dateString) return 'Unknown';
    try {
      const date = new Date(dateString);
      return date.toLocaleString();
    } catch (error) {
      return dateString;
    }
  };

  // Create a simplified object suitable for table display
  const formattedStats = {
    'Last Update': formatDate(stats.lastUpdate),
    'Total Rules': stats.totalRules.toLocaleString(),
    'Database Health': `${stats.databaseHealth.contentPercentage}% Complete`,
    
    // OS breakdown
    'Windows Rules': stats.operatingSystems.windows.toLocaleString(),
    'Linux Rules': stats.operatingSystems.linux.toLocaleString(),
    'macOS Rules': stats.operatingSystems.macos.toLocaleString(),
    'Other OS Rules': stats.operatingSystems.other.toLocaleString(),
    
    // Add severity levels
    ...(stats.severityLevels || []).reduce((acc, level) => {
      const levelName = level.level 
        ? level.level.charAt(0).toUpperCase() + level.level.slice(1) 
        : 'Unknown';
      
      acc[`${levelName} Severity`] = level.count.toLocaleString();
      return acc;
    }, {})
  };
  
  // Add top MITRE tactics if available
  if (stats.mitreTactics && stats.mitreTactics.length > 0) {
    stats.mitreTactics.forEach((tactic, index) => {
      if (index < 5) { // Only include top 5 for brevity
        const formattedTactic = tactic.tactic
          .replace(/-/g, ' ')
          .split(' ')
          .map(word => word.charAt(0).toUpperCase() + word.slice(1))
          .join(' ');
          
        formattedStats[`MITRE: ${formattedTactic}`] = tactic.count.toLocaleString();
      }
    });
  }
    
  return formattedStats;
}

/**
 * Format Sigma search results for CLI display
 * 
 * @param {Object} searchResults - The search results object
 * @returns {Object} Formatted results ready for CLI display
 */
function formatSigmaSearchResults(searchResults) {
  if (!searchResults || !searchResults.results) {
    return { error: 'No search results available' };
  }
  
  // Return a structure with results and meta info
  return {
    results: searchResults.results.map(rule => ({
      id: rule.id || '',
      title: rule.title || '',
      author: rule.author || 'Unknown',
      level: rule.level || 'medium'
    })),
    totalCount: searchResults.totalCount || 0
  };
}

/**
 * Format Sigma rule details for CLI display
 * 
 * @param {Object} ruleDetails - The rule details object
 * @returns {Object} Formatted details ready for CLI display
 */
function formatSigmaDetails(ruleDetails) {
  if (!ruleDetails) {
    return { error: 'No rule details available' };
  }
  
  // Filter and format the rule details for CLI display
  const formattedDetails = {};
  
  // Include only the most important fields for display
  const fieldsToInclude = [
    'id', 'title', 'description', 'status', 'author',
    'level', 'falsepositives', 'references',
    'created', 'modified'
  ];
  
  // Add detection information if available
  if (ruleDetails.detection && ruleDetails.detection.condition) {
    fieldsToInclude.push('detection_condition');
    formattedDetails['detection_condition'] = ruleDetails.detection.condition;
  }
  
  // Add logsource information if available
  if (ruleDetails.logsource) {
    if (ruleDetails.logsource.product) {
      fieldsToInclude.push('logsource_product');
      formattedDetails['logsource_product'] = ruleDetails.logsource.product;
    }
    
    if (ruleDetails.logsource.category) {
      fieldsToInclude.push('logsource_category');
      formattedDetails['logsource_category'] = ruleDetails.logsource.category;
    }
    
    if (ruleDetails.logsource.service) {
      fieldsToInclude.push('logsource_service');
      formattedDetails['logsource_service'] = ruleDetails.logsource.service;
    }
  }
  
  // Format date fields
  const dateFields = ['created', 'modified'];
  
  for (const [key, value] of Object.entries(ruleDetails)) {
    if (fieldsToInclude.includes(key)) {
      // Format dates
      if (dateFields.includes(key) && value) {
        try {
          formattedDetails[key] = new Date(value).toLocaleString();
        } catch (e) {
          formattedDetails[key] = value;
        }
      } 
      // Format arrays
      else if (Array.isArray(value)) {
        formattedDetails[key] = value.join(', ');
      }
      // Default handling
      else {
        formattedDetails[key] = value;
      }
    }
  }
  
  return formattedDetails;
}

module.exports = {
  formatSigmaStats,
  formatSigmaSearchResults,
  formatSigmaDetails
};